# Zeedhi Tools #

[![project building badge](http://192.168.122.56:9090/view/Zeedhi/job/tools/badge/icon)](http://192.168.122.56:9090/view/Zeedhi/job/tools/)

## Descrição ##
Zeedhi Tools é uma ferramenta para auxiliar os processos de importar as tabelas do seu banco para os meta-dados e geração de entidades para o [Doctrine ORM](http://www.doctrine-project.org/projects/orm.html). É recomendado que você tenha algum conhecimento sobre o funcionamento e uso do Doctrine ORM para utilização das entidades, leia pelo menos o [Getting Started](http://docs.doctrine-project.org/projects/doctrine-orm/en/latest/tutorials/getting-started.html).

## Como instalar ##
Adicione o nosso [repositório de pacotes](http://cdn.zeedhi.com/composer/), ao seu arquivo "composer.json", para mais detalhes [aqui](https://getcomposer.org/doc/04-schema.md#repositories)!
```json
{
    "repositories": [
        {
            "type": "composer",
            "url": "http://cdn.zeedhi.com/composer"
        }
    ]
}
```
Rode o comando `composer require --dev zeedhi/tools:0.1.x-dev` no mesmo diretório onde se encontra o "composer.json" do seu projeto, para mais [detalhes](https://getcomposer.org/doc/03-cli.md#require). Ou adicione a linha `"zeedhi/tools": "0.1.x-dev"`, dentro da propriedade "require-dev" ao seu arquivo composer.json e rode o `composer update`.

## Configuração ##

Para utilizar o zeedhi/tools/generator, processo que importa suas tabelas para o metadadados e gera suas entidades é preciso criar um arquivo de configuração, nomeado `entities.json` seguindo o modelo abaixo, copiado do zeedhi entities.

```
{
    "mappingType"    : "xml",
    "mappingPath"    : "dcm",
    "srcPath"        : "src",
    "connection" : {
        "driver"     : "oci8",
        "host"       : "192.168.122.55",
        "port"       : "1521",
        "username"   : "ZEEDHI",
        "password"   : "GQvJIzVzLT28fvte1l2552XLloZAIbb0BZjM8apaed0=",
        "dbname"     : "xe"
    },
    "namespace"      : "Namespace\\SeparadoPor\\BarrasDuplas",
    "inflectorClassName" : "\\Nome\\DaSua\\Classe\\Inflector",
    "subModelsByTables" : {
        "table_name" : "SUB_MODEL_NAME",
    }
}
```

Parametros do arquivo de configuração:

| Parametro             | Descrição                                                 |
| --------------------- | --------------------------------------------------------- |
| metaConnection        | Sua conexão com os metadados do Zeedhi.                   |
| workConnection        | Conexão onde será lido o schema para gerar as entidades   |
| srcPath               | Caminho onde será gerado os arquivos php                  |
| mappingPath           | Caminho onde será gerado que mapeam as entidades          |
| mappingType           | Tipo do mapeamento gerado, "xml", "yaml", annotation"     |
| connectionId          | Id do metadado da sua conexão.                            |
| organizationId        | Id do metadado da sua organização.                        |
| inflectorClassName    | Detalhada na sessão a seguir.                             |
| namespace             | Namespace pai das suas entidades                          |
| subModelsByTables     | Map de submodelos, indexadas pelo nome das tabelas        |
| userId                | Parametro depreciado com o advento do login.              |
| skipEntitiesRelations | Pular a geração de relacionamento nas entidades.          |

### Inflector ###

O Inflector é a classe que define como os nomes de tabelas e colunas, são transformados em nomes de classes, propriedades (tanto para fields e relações) e métodos. Todo inflector deve ser instanciado sem passagem de parametros e implementar a interface `\Zeedhi\Tools\EntityGenerator\Inflector`, que demanda os seguintes métodos:
* _classify_: Recebe o nome da tabela e deve retornar o nome da classe.
* _propertify_: Recebe o nome da coluna e deve retornar o nome do field.
* _methodfy_: Recebe o nome field e deve retornar o do método.
* _relationify_: Recebe o nome do field escolhido a representar o relacionamento e deve retornar o nome definitivo para o field a ser gerado.

Existe uma implementação default do inflector chamada [`\Zeedhi\Tools\EntityGenerator\DefaultInflector`](http://192.168.122.56/zeedhi/tools/blob/master/src/Zeedhi/Tools/EntityGenerator/DefaultInflector.php) que pode ser usada diretamente caso você não deseje implementar um próprio.
Perceba que o nome da classe gerado seguirá a seguinte estrutura, onde os negritos são parametros do arquivo de configuração:
> \\**namespace**\Inflector::classify(**subModelsByTables**)\Entities\Inflector::classify(**table_name**)

## Uso Básico ##
Para gerar as entidades e metadados você precisa rodar o comando `vendor\zeedhi\tools\generator`, em seguida sera questionado sobre o seu usuário e senha, após este tempo ele iria começar a gerar as entidades e imprimir mensagens sobre o andamento do processo. O generator somente irá considerar as tabelas listadas na propriedade `subModelsByTable` do arquivo `entities.json`. Caso o gerador conclua com sucesso aparecerá a mensagem "Finishing him!".

## Uso Avançado ##
É comum ter que regerar apenas uma, ou um pequeno grupo de entidades/tabelas, para tal efeito basta executar o comando `vendor\zeedhi\tools\generator TABELA_1 TABELA_2 ... TABELA_N`. Desta maneira ele so lerá e gerará as tabelas selecionadas, note que elas ainda precisam serem listadas na propriedade `subModelsByTable` do arquivo `entities.json`.

É possível gerar as entidades sem se preocupar com inversões de dependências. Para tal, basta executar o generator com a flag `--clean`.

Pode ser necessário ter um output mais verboso para entender e debugar os seus erros e problemas com as suas tabelas, ou mesmo para entender os passos que estão sendo feitos pelo tools, para tal, use a flag `--verbose`.

### Oracle Wallet
Você pode definir conexão usando oracle wallet usando a configuração como a seguir:

```js
    "connection" : {
        "driver"     : "oci8",
        "username"   : "/",
        "password"   : "",
        "dbname"     : "/TEKNISA_SAAS_REFERENCE",
        "isOracleWallet" : true
    },
```


**OBS: Se quiser saber mais recomendo que leia o código fonte! Ele está relativamente bem escrito.**

**OBS - 2: O(s) nome(s) da(s) tables(s) deve(m) vir como ultimo argumento.**

## Erros possíveis ##
* `[error] TABLE_NAME was not found in current database`:  A tabela foi listada, junto a algum submodel, mas não se encontra na base. Pode ter sido escolhida a base errada, existir um erro de digitação no nome da tabela.
* `[warning] Tab TABLE_NAME has not a primary key`: Não foi encontrado primary key para a tabela relacionada. A tabela será importada normalmente, mas não terá seus arquivos de entidades gerados.
* `[error] Constraint FK_CONSTRAINT_NAME reference a pk PK_CONSTRAINT_NAME non imported`: A tabela que contém a `FK_CONSTRAINT_NAME` referência a uma tabela que ainda não foi importada. Provavelmente a tabela não importada não é listada no arquivo `entities.json` ou não foi importada porque é nova e a geração atual usou o filtro de tabelas, desconsiderando assim as novas tabelas do arquivo de configuração. Quando isto ocorre a FK não é importada e portanto o relacionamento entre as entidades não é geradao. Para consertar você pode adicionar a nova tabela no processo e mandar gerar as entidades de novo.
* `[warning] Table ARQMAGPISCOFINS skipped it has no primary key! `: Está mensagem é uma consequencia do `Tab TABLE_NAME has not a primary key`, porém no momento de gerar os metadados das entidades.

## FAQ ##
_OBS: Nenhuma pergunta foi feita ainda! Mas para realizar novas perguntas sugiro que crie uma ["issue"](http://192.168.122.56/zeedhi/tools/issues) dentro do GitLab, e me marque, [@mateuslopes](http://192.168.122.56/u/mateus.lopes), que terei o prazer de lhe responder, e talvez adicionar sua pergunta a este documento._

_OBS: Sinta-se livre para sugerir novas funcionalidades. Melhor sinta-se livre para submeter [MergeRequests](http://192.168.122.56/zeedhi/migration/tools) com elas. Mas eu me sinto livre para recusa-las caso ache que tais implementações fujam do propósito do projeto.
